# -*- coding: utf-8 -*-
try:
    from plone.rfc822.defaultfields import BaseFieldMarshaler
    HAVE_MARSHALER = True
except ImportError:
    HAVE_MARSHALER = False

if HAVE_MARSHALER:

    from email.Encoders import encode_base64
    from plone.namedfile import NamedBlobFile
    from plone.namedfile import NamedBlobImage
    from plone.namedfile import NamedFile
    from plone.namedfile import NamedImage
    from plone.namedfile.interfaces import INamedBlobFileField
    from plone.namedfile.interfaces import INamedBlobImageField
    from plone.namedfile.interfaces import INamedFileField
    from plone.namedfile.interfaces import INamedImageField
    from zope.component import adapter
    from zope.interface import Interface

    class BaseNamedFileFieldMarshaler(BaseFieldMarshaler):
        """Base marshaler for plone.namedfile values. Actual adapters are
        registered as subclasses.
        """

        ascii = False
        factory = None

        def encode(self, value, charset='utf-8', primary=False):
            # we only support encoding a file value in the body of a message,
            # never in a header
            if not primary:
                raise ValueError(
                    'File fields can only be marshaled as primary fields')
            if value is None:
                return None
            return value.data

        def decode(
                self,
                value,
                message=None,
                charset='utf-8',
                contentType=None,
                primary=False):
            filename = None
            if primary and message is not None:
                filename = message.get_filename(None)
            return self.factory(value, contentType or '', filename)

        def getContentType(self):
            value = self._query()
            if value is None:
                return None
            return value.contentType

        def getCharset(self, default='utf-8'):
            return None

        def postProcessMessage(self, message):
            """Encode message as base64 and set content disposition
            """
            value = self._query()
            if value is not None:
                filename = value.filename
                if filename:
                    message.add_header('Content-Disposition', 'attachment')
                    message.set_param(
                        'filename',
                        filename.encode('utf-8'),
                        header='Content-Disposition',
                        charset='utf-8'
                    )

            encode_base64(message)

    @adapter(Interface, INamedFileField)
    class NamedFileFieldMarshaler(BaseNamedFileFieldMarshaler):
        """Marshaler for an INamedFile field
        """
        factory = NamedFile

    @adapter(Interface, INamedImageField)
    class NamedImageFieldMarshaler(BaseNamedFileFieldMarshaler):
        """Marshaler for an INamedImage field
        """
        factory = NamedImage

    @adapter(Interface, INamedBlobFileField)
    class NamedBlobFileFieldMarshaler(BaseNamedFileFieldMarshaler):
        """Marshaler for an INamedBlobFile field
        """
        factory = NamedBlobFile

    @adapter(Interface, INamedBlobImageField)
    class NamedBlobImageFieldMarshaler(BaseNamedFileFieldMarshaler):
        """Marshaler for an INamedBlobImage field
        """
        factory = NamedBlobImage



# -*- coding: utf-8 -*-
from plone.namedfile.interfaces import IBlobby

import mimetypes
import os.path
import urllib


try:
    # use this to stream data if we can
    from ZPublisher.Iterators import filestream_iterator
except ImportError:
    filestream_iterator = None


def safe_basename(filename):
    """Get the basename of the given filename, regardless of which platform
    (Windows or Unix) it originated from.
    """
    fslice = max(
        filename.rfind('/'),
        filename.rfind('\\'),
        filename.rfind(':'),
    ) + 1
    return filename[fslice:]


def get_contenttype(
        file=None,
        filename=None,
        default='application/octet-stream'):
    """Get the MIME content type of the given file and/or filename.
    """

    file_type = getattr(file, 'contentType', None)
    if file_type:
        return file_type

    filename = getattr(file, 'filename', filename)
    if filename:
        extension = os.path.splitext(filename)[1].lower()
        return mimetypes.types_map.get(extension, 'application/octet-stream')

    return default


def set_headers(file, response, filename=None):
    """Set response headers for the given file. If filename is given, set
    the Content-Disposition to attachment.
    """

    contenttype = get_contenttype(file)

    response.setHeader('Content-Type', contenttype)
    response.setHeader('Content-Length', file.getSize())

    if filename is not None:
        if not isinstance(filename, unicode):
            filename = unicode(filename, 'utf-8', errors='ignore')
        filename = urllib.quote(filename.encode('utf8'))
        response.setHeader(
            'Content-Disposition',
            'attachment; filename*=UTF-8\'\'{0}'.format(filename)
        )


def stream_data(file):
    """Return the given file as a stream if possible.
    """

    if IBlobby.providedBy(file):
        if file._blob._p_blob_uncommitted:
            return file.data
        if filestream_iterator is not None:
            return filestream_iterator(file._blob.committed(), 'rb')

    return file.data



# -*- coding: utf-8 -*-
from plone.namedfile.file import NamedBlobFile    # noqa
from plone.namedfile.file import NamedBlobImage   # noqa
from plone.namedfile.file import NamedFile        # noqa
from plone.namedfile.file import NamedImage       # noqa



# -*- coding: utf-8 -*-
"""Copy hook for proper copying blob data
"""
# This file was borrowed from z3c.blobfile and is licensed under the terms of
# the ZPL.
from plone.namedfile.interfaces import INamedBlobFile
from ZODB.blob import Blob
from zope.component import adapter
from zope.copy.interfaces import ICopyHook
from zope.copy.interfaces import ResumeCopy
from zope.interface import implementer

import shutil


@implementer(ICopyHook)
@adapter(INamedBlobFile)
class BlobFileCopyHook(object):
    """A copy hook that fixes the blob after copying"""

    def __init__(self, context):
        self.context = context

    def __call__(self, toplevel, register):
        register(self._copyBlob)
        raise ResumeCopy

    def _copyBlob(self, translate):
        target = translate(self.context)
        target._blob = Blob()
        fsrc = self.context._blob.open('r')
        fdst = target._blob.open('w')
        shutil.copyfileobj(fsrc, fdst)
        fdst.close()
        fsrc.close()
